@@ -32,14 +32,31 @@ def infinite_lazylist(fn):
     return wrapped
 
 
+def lazylist_from(iterable):
+    def dec(fn):
+        def wrapped(*args, **kwargs):
+            return LazyList(
+                fn(*args, **kwargs),
+                isinf=(type(iterable) is LazyList and iterable.infinite),
+            )
+
+        wrapped.__name__ = fn.__name__
+
+        return wrapped
+
+    return dec
+
+
 class LazyList:
     def __add__(self, rhs):
-        @lazylist
         def gen():
             yield from self
             yield from rhs
 
-        return gen()
+        return LazyList(
+            gen(),
+            isinf=(self.infinite or (type(rhs) is LazyList and rhs.infinite)),
+        )
 
     def __bool__(self):
         try:
@@ -94,7 +111,7 @@ class LazyList:
             )
             if stop is None:
 
-                @lazylist
+                @lazylist_from(self)
                 def infinite_index():
                     i = start or 0
                     if i >= 0:
@@ -120,7 +137,11 @@ class LazyList:
                 return ret
         else:
             if position < 0:
-                self.generated += list(self)
+                while True:
+                    try:
+                        next(self)
+                    except StopIteration:
+                        break
                 return self.generated[position]
             elif position < len(self.generated):
                 return self.generated[position]
@@ -214,12 +235,15 @@ class LazyList:
         temp = self.listify()
         return temp.count(other)
 
-    @lazylist
     def filter(self, fn):
-        """A `LazyList` containing only elements for whom `fn` is true"""
-        for item in self:
-            if fn(item):
-                yield item
+        @lazylist_from(self)
+        def f():
+            """A `LazyList` containing only elements for whom `fn` is true"""
+            for item in self:
+                if fn(item):
+                    yield item
+
+        return f()
 
     def has_ind(self, ind: int):
         """Whether or not this list is long enough for index `ind`"""
@@ -255,9 +279,15 @@ class LazyList:
 
         vy_print(open, "", ctx=ctx)
         for lhs in self.generated[:-1]:
-            vy_print(lhs, sep, ctx=ctx)
+            if isinstance(lhs, (types.FunctionType, LazyList)):
+                vy_print(lhs, sep, ctx=ctx)
+            else:
+                vy_print(vy_repr(lhs, ctx), sep, ctx=ctx)
         if self.generated:
-            vy_print(self.generated[-1], "", ctx=ctx)
+            if isinstance(self.generated[-1], (types.FunctionType, LazyList)):
+                vy_print(self.generated[-1], "", ctx=ctx)
+            else:
+                vy_print(vy_repr(self.generated[-1], ctx), "", ctx=ctx)
 
         try:
             lhs = next(self)
@@ -273,10 +303,13 @@ class LazyList:
         except StopIteration:
             vy_print(close, end, ctx=ctx)
 
-    @lazylist
     def appended(self, value):
-        yield from self
-        yield value
+        @lazylist_from(self)
+        def gen():
+            yield from self
+            yield value
+
+        return gen()
 
     @lazylist
     def reversed(self):
