@@ -8,6 +8,7 @@ from vyxal import encoding, helpers, lexer, parse, structure
 from vyxal.elements import *
 from vyxal.helpers import indent_str, uncompress
 from vyxal.lexer import Token, TokenType
+from vyxal.encoding import utf8_to_vyxal
 
 NILADIC_TYPES = (
     TokenType.STRING,
@@ -18,6 +19,9 @@ NILADIC_TYPES = (
     TokenType.CODEPAGE_NUMBER,
 )
 
+# TranspilationOptions has been moved to context.py
+from vyxal.context import TranspilationOptions
+
 
 def lambda_wrap(
     branch: list[vyxal.structure.Structure],
@@ -48,25 +52,26 @@ def lambda_wrap(
 
 def transpile(
     program: str,
-    dict_compress: bool = True,
-    variables_as_digraphs: bool = False,
+    options: TranspilationOptions = TranspilationOptions(),
 ) -> str:
     return transpile_ast(
-        vyxal.parse.parse(vyxal.lexer.tokenise(program, variables_as_digraphs)),
-        dict_compress=dict_compress,
+        vyxal.parse.parse(
+            vyxal.lexer.tokenise(program, options.variables_as_digraphs)
+        ),
+        options=options,
     )
 
 
 def transpile_ast(
     program: list[vyxal.structure.Structure],
     indent: int = 0,
-    dict_compress: bool = True,
+    options: TranspilationOptions = TranspilationOptions(),
 ) -> str:
     """Transpile a given program (as a parsed list of structures) into Python"""
     if not program:
         return helpers.indent_str("pass", indent)
     return "\n".join(
-        transpile_single(struct, indent=indent, dict_compress=dict_compress)
+        transpile_single(struct, indent=indent, options=options)
         for struct in program
     )
 
@@ -74,17 +79,13 @@ def transpile_ast(
 def transpile_single(
     token_or_struct: Union[Token, vyxal.structure.Structure],
     indent: int,
-    dict_compress: bool = True,
+    options: TranspilationOptions,
 ) -> str:
     """Transpile a single token or structure"""
     if isinstance(token_or_struct, Token):
-        return transpile_token(
-            token_or_struct, indent, dict_compress=dict_compress
-        )
+        return transpile_token(token_or_struct, indent, options=options)
     elif isinstance(token_or_struct, vyxal.structure.Structure):
-        return transpile_structure(
-            token_or_struct, indent, dict_compress=dict_compress
-        )
+        return transpile_structure(token_or_struct, indent, options=options)
     raise ValueError(
         "Input must be a Token or Structure,"
         f" was {type(token_or_struct).__name__}: {token_or_struct}"
@@ -92,12 +93,12 @@ def transpile_single(
 
 
 def transpile_token(
-    token: Token, indent: int, dict_compress: bool = True
+    token: Token, indent: int, options: TranspilationOptions
 ) -> str:
     """Transpile a single token"""
     if token.name == TokenType.STRING:
         # Make sure we avoid any ACE exploits
-        if dict_compress:
+        if options.dict_compress and not options.utf8strings:
             string = uncompress(token)
         else:
             string = token.value
@@ -120,10 +121,13 @@ def transpile_token(
                 temp += '\\"'
             elif char == "\n":
                 temp += "\\n"
-            elif char == "Π":
+            elif char == "Π" and not options.utf8strings:
                 temp += '" + vy_str(pop(stack, 1, ctx), ctx=ctx) + "'
             else:
                 temp += char
+        if options.utf8strings:
+            # Since all string-related stuff is ASCII this shouldn't ACE
+            temp = bytes(map(ord, utf8_to_vyxal(temp))).decode("utf-8")
         return indent_str(f'stack.append("{temp}")', indent)
     elif token.name == TokenType.NUMBER:
         parts = [
@@ -176,13 +180,13 @@ def transpile_token(
 
 
 def transpile_structure(
-    struct: vyxal.structure.Structure, indent: int, dict_compress: bool = True
+    struct: vyxal.structure.Structure,
+    indent: int,
+    options: TranspilationOptions,
 ) -> str:
     """Transpile a single vyxal.structure."""
     if isinstance(struct, vyxal.structure.GenericStatement):
-        return transpile_single(
-            struct.branches[0][0], indent, dict_compress=dict_compress
-        )
+        return transpile_single(struct.branches[0][0], indent, options=options)
     if isinstance(struct, vyxal.structure.IfStatement):
         # Holds indentation level as elifs will be nested inside the else part
         new_indent = indent
@@ -197,23 +201,17 @@ def transpile_structure(
                 cond = struct.branches[i]
                 res += indent_str("else:", new_indent)
                 new_indent += 1
-                res += transpile_ast(
-                    cond, new_indent, dict_compress=dict_compress
-                )
+                res += transpile_ast(cond, new_indent, options=options)
 
             res += indent_str("condition = pop(stack, 1, ctx=ctx)", new_indent)
             res += indent_str("if boolify(condition, ctx):", new_indent)
-            res += transpile_ast(
-                body, new_indent + 1, dict_compress=dict_compress
-            )
+            res += transpile_ast(body, new_indent + 1, options=options)
 
         # There's an extra else body at the end
         if len(struct.branches) % 2 == 0:
             body = struct.branches[-1]
             res += indent_str("else:", new_indent)
-            res += transpile_ast(
-                body, new_indent + 1, dict_compress=dict_compress
-            )
+            res += transpile_ast(body, new_indent + 1, options=options)
 
         return res
     if isinstance(struct, vyxal.structure.ForLoop):
@@ -231,26 +229,20 @@ def transpile_structure(
                 indent,
             )
             + indent_str(f"    ctx.context_values.append({var})", indent)
-            + transpile_ast(
-                struct.body, indent + 1, dict_compress=dict_compress
-            )
+            + transpile_ast(struct.body, indent + 1, options=options)
             + indent_str("    ctx.context_values.pop()", indent)
         )
     if isinstance(struct, vyxal.structure.WhileLoop):
         return (
-            transpile_ast(struct.condition, indent, dict_compress=dict_compress)
+            transpile_ast(struct.condition, indent, options=options)
             + indent_str("condition = pop(stack, 1, ctx=ctx)", indent)
             + indent_str("counter = ctx.range_start", indent)
             + indent_str("while boolify(condition, ctx):", indent)
             + indent_str("    ctx.context_values.append(condition)", indent)
             + indent_str("    ctx.context_values.append(counter)", indent)
-            + transpile_ast(
-                struct.body, indent + 1, dict_compress=dict_compress
-            )
+            + transpile_ast(struct.body, indent + 1, options=options)
             + indent_str("    ctx.context_values.pop()", indent)
-            + transpile_ast(
-                struct.condition, indent + 1, dict_compress=dict_compress
-            )
+            + transpile_ast(struct.condition, indent + 1, options=options)
             + indent_str("    condition = pop(stack, 1, ctx=ctx)", indent)
             + indent_str("    counter += 1", indent)
         )
@@ -300,7 +292,7 @@ def transpile_structure(
             + indent_str("ctx.inputs.append([parameters[::-1], 0])", indent + 1)
             + indent_str(f"this = VAR_{var}", indent + 1)
             + indent_str(
-                transpile_ast(struct.body, dict_compress=dict_compress),
+                transpile_ast(struct.body, options=options),
                 indent + 1,
             )
             + indent_str("ctx.context_values.pop()", indent + 1)
@@ -317,24 +309,24 @@ def transpile_structure(
             + transpile_lambda(
                 struct.lam,
                 indent,
-                dict_compress=dict_compress,
+                options=options,
                 extra="lhs = pop(stack, 1, ctx); stack += iterable(lhs, ctx=ctx)",
             )
             + transpile_token(
                 Token(TokenType.GENERAL, struct.after),
                 indent,
-                dict_compress=dict_compress,
+                options=options,
             )
         )
     if isinstance(struct, vyxal.structure.Lambda):
-        return transpile_lambda(struct, indent, dict_compress=dict_compress)
+        return transpile_lambda(struct, indent, options=options)
     if isinstance(struct, vyxal.structure.LambdaOp):
         return transpile_lambda(
-            struct.lam, indent, dict_compress=dict_compress
+            struct.lam, indent, options=options
         ) + transpile_token(
             Token(TokenType.GENERAL, struct.after),
             indent,
-            dict_compress=dict_compress,
+            options=options,
         )
 
     if isinstance(struct, vyxal.structure.ListLiteral):
@@ -347,7 +339,7 @@ def transpile_structure(
                 temp += (
                     indent_str("def list_item(s, ctx):", indent)
                     + indent_str("stack = list(deep_copy(s))", indent + 1)
-                    + transpile_ast(x, indent + 1, dict_compress=dict_compress)
+                    + transpile_ast(x, indent + 1, options=options)
                     + indent_str("if len(stack) == 0: return", indent + 1)
                     + indent_str("return pop(stack, 1, ctx=ctx)", indent + 1)
                     + indent_str("f = list_item(stack, ctx)", indent)
@@ -363,11 +355,17 @@ def transpile_structure(
         element_A = transpile_ast(
             [lambda_wrap([struct.function_A])],
             indent,
-            dict_compress=dict_compress,
+            options=options,
+        )
+
+        eager = isinstance(
+            struct.function_A,
+            (vyxal.structure.RecurseStatement, vyxal.structure.BreakStatement),
         )
         return (
             element_A
             + "\n"
+            + indent_str("eager = " + str(eager), indent)
             + indent_str("function_A = pop(stack, 1, ctx)", indent)
             + indent_str(modifiers.get(struct.modifier, "pass"), indent)
         )
@@ -376,12 +374,12 @@ def transpile_structure(
         element_A = transpile_ast(
             [lambda_wrap([struct.function_A])],
             indent,
-            dict_compress=dict_compress,
+            options=options,
         )
         element_B = transpile_ast(
             [lambda_wrap([struct.function_B])],
             indent,
-            dict_compress=dict_compress,
+            options=options,
         )
         return (
             element_A
@@ -396,17 +394,17 @@ def transpile_structure(
         element_A = transpile_ast(
             [lambda_wrap([struct.function_A])],
             indent,
-            dict_compress=dict_compress,
+            options=options,
         )
         element_B = transpile_ast(
             [lambda_wrap([struct.function_B])],
             indent,
-            dict_compress=dict_compress,
+            options=options,
         )
         element_C = transpile_ast(
             [lambda_wrap([struct.function_C])],
             indent,
-            dict_compress=dict_compress,
+            options=options,
         )
         return (
             element_A
@@ -477,7 +475,7 @@ def transpile_structure(
 def transpile_lambda(
     lam: vyxal.structure.Lambda,
     indent: int,
-    dict_compress: bool = True,
+    options: TranspilationOptions,
     extra: str = "",
 ):
     id_ = secrets.token_hex(16)
@@ -531,7 +529,7 @@ def transpile_lambda(
         + indent_str("ctx.stacks.append(stack);", indent + 1)
         + (extra and indent_str(extra, indent + 1))
         + indent_str(
-            transpile_ast(lam.body, dict_compress=dict_compress),
+            transpile_ast(lam.body, options=options),
             indent + 1,
         )
         + indent_str(
